@@ -14,7 +14,8 @@ FileSystemModel::FileSystemModel(QObject *parent) :
     firstDetail = Size;
     secondDetail = Date;
 
-    enableDotDot(false);
+    showThumbnails = false;
+    showDotDot = false;
     enableHidden(false);
 }
 
@@ -54,6 +55,7 @@ void FileSystemModel::reload()
     // Avoid pointless reloading before the first path is set
     if (currentDir.path() == ".") return;
 
+    emit loadingStarted();
     itemBuffer.clear();
     model->clear();
 
@@ -76,6 +78,8 @@ void FileSystemModel::reload()
 
         currentWorker = worker;
         worker->start();
+    } else {
+        emit loadingFinished();
     }
 }
 
@@ -99,6 +103,7 @@ void FileSystemModel::onWorkerFinished()
         currentWorker = NULL;
         while (!itemBuffer.isEmpty())
             model->appendRow(itemBuffer.takeFirst());
+        emit loadingFinished();
     } else {
         workers.removeOne(static_cast<FileSystemModelWorker*>(this->sender()));
     }
@@ -108,8 +113,20 @@ void FileSystemModel::onWorkerFinished()
 
 void FileSystemModel::enableDotDot(bool enable)
 {
+    if (showDotDot != enable && currentDir.path() != "/") {
+        if (enable) {
+            QStandardItem *item = new QStandardItem();
+            item->setText("..");
+            item->setIcon(QIcon::fromTheme("filemanager_folder_up"));
+            item->setData(currentDir.absolutePath() + "/..", PathRole);
+            item->setFlags(item->flags() & ~Qt::ItemIsSelectable);
+            model->insertRow(0, item);
+        } else {
+            model->removeRow(0);
+        }
+    }
+
     showDotDot = enable;
-    reload();
 }
 
 void FileSystemModel::enableHidden(bool enable)
