@@ -106,11 +106,11 @@ void LocationWidget::emitLocationFromEditor()
         emit locationSelected(locationEdit->text());
 }
 
-void LocationWidget::setLocation(QString path)
+int LocationWidget::setLocation(QString path, int oldPosition)
 {
     currentPath = path;
 
-    disableEditor();
+    int newPosition = 0;
 
     QLayout *buttonLayout = buttonWidget->layout();
     QStringList pathParts = path.split('/', QString::SkipEmptyParts);
@@ -128,32 +128,47 @@ void LocationWidget::setLocation(QString path)
                 // The new path is a substring of the old path so far
                 if (i == pathParts.size()-1) {
                     // Finished without mismatches
-                    button->setCheckable(true);
-                    button->setChecked(true);
-                    checkedButton = button;
+                    if (button == checkedButton) {
+                        newPosition = oldPosition;
+                    } else {
+                        button->setCheckable(true);
+                        button->setChecked(true);
+                        checkedButton = button;
+                        newPosition = scrollPositions.at(i);
+                    }
                 } else {
                     // Make sure that no unnecessary buttons on the way are highlighted
-                    button->setChecked(false);
-                    button->setCheckable(false);
+                    if (button->isChecked()) {
+                        button->setChecked(false);
+                        button->setCheckable(false);
+                        scrollPositions[i] = oldPosition;
+                    }
                 }
             } else {
                 // Remove all mismatching buttons
-                while (buttonLayout->itemAt(i))
+                while (buttonLayout->itemAt(i)) {
                     delete buttonLayout->takeAt(i)->widget();
+                    scrollPositions.removeAt(i);
+                }
                 break;
             }
         } else {
             // The path is already parsed, so only uncheck the remaining buttons
-            button->setChecked(false);
-            button->setCheckable(false);
+            if (button->isChecked()) {
+                button->setChecked(false);
+                button->setCheckable(false);
+                scrollPositions[i] = oldPosition;
+            }
         }
     }
 
     // Add more buttons if necessary
     if (i < pathParts.size()) {
         // Remaining buttons for the new path, except the last one
-        while(i < pathParts.size()-1)
+        while(i < pathParts.size()-1) {
             buttonLayout->addWidget(createButton(pathParts.at(i++)));
+            scrollPositions.append(0);
+        }
 
         // Add and highlight the last button
         QAbstractButton *lastButton = createButton(pathParts.last());
@@ -161,9 +176,12 @@ void LocationWidget::setLocation(QString path)
         lastButton->setChecked(true);
         checkedButton = lastButton;
         buttonLayout->addWidget(lastButton);
+        scrollPositions.append(0);
     }
 
     disableEditor();
+
+    return newPosition;
 }
 
 void LocationWidget::resizeEvent(QResizeEvent *e)
